
/* This file prints the contents of challenge and instance files.
 * The path to a .challenge file must be provided; the path to a corresponding
 * instance file may also be provided. The challenge contents are printed, and
 * the instance contents are printed if a path was provided.
 * See the code for examples of how to access message contents.
 *
 * More information about parsing can be found at
 * https://developers.google.com/protocol-buffers/docs/reference/java-generated
 */
// The included jar file was obtained from http://search.maven.org/remotecontent?filepath=com/google/protobuf/protobuf-java/2.6.1/protobuf-java-2.6.1.jar

import crypto.proto.RLWE.challenges.*;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.PrintStream;

class ChallInstParser {
	public static void main(String[] args) throws Exception {

		// read a challenge file
		Challenges.Challenge chall =
			Challenges.Challenge.parseFrom(new FileInputStream(args[0]));
		System.out.println(chall); // print all contents
		// some examples showing how to access message contents

    // number of instances in the challenge
		System.out.println(chall.getNumInstances());
    // an enum showing which type of parameter the message contains
    System.out.println(chall.getParamsCase());
		switch (chall.getParamsCase()) {
			case CPARAMS: {
        // cyclotomic index of a challenge
        System.out.println(chall.getCparams().getM());
        break;
      }
      case DPARAMS: {
        // number of samples in each instance
        System.out.println(chall.getDparams().getNumSamples());
        break;
      }
      case RPARAMS: {
        // input modulus of the RLWR challenge
        System.out.println(chall.getRparams().getQ());
        break;
      }
		}

		if (args.length == 2) {
			// also read an instance file
			if (chall.hasCparams()) {
				Challenges.InstanceCont inst =
					Challenges.InstanceCont.parseFrom(new FileInputStream(args[1]));
				System.out.println(inst); // print all contents
				// it is easy to access any member of the parsed message
        /* each instance has many samples (print something from the first sample)
         * each sample is a pair (a,b=a*s+e) (print something in the ring element 'a')
         * each ring element is represented as a list of coefficients with resepct to
         * the decoding basis. (print the first coefficient)
         */
        System.out.println(inst.getSamples(0).getA().getXs(0));
			}
			else if (chall.hasDparams()) {
				Challenges.InstanceDisc inst =
					Challenges.InstanceDisc.parseFrom(new FileInputStream(args[1]));
				System.out.println(inst); // print all contents
        System.out.println(inst.getSamples(0).getA().getXs(0));
			}
			else if (chall.hasRparams()) {
				Challenges.InstanceRLWR inst =
					Challenges.InstanceRLWR.parseFrom(new FileInputStream(args[1]));
				System.out.println(inst); // print all contents
        System.out.println(inst.getSamples(0).getA().getXs(0));
			}
			else {
				System.out.println("Challenge parse error.");
				System.exit(0);
			}
		}
    System.out.println("See source code in ChallInstParser.java for examples of how to access specific message members.");
	}
}
